#include "_Model.h"

void _strTM::LoadSettings(const char* filename)
{	
	FILE* fp;
	fp = fopen(filename, "r");
	fscanf(fp, "TOPIC:%d ", &m_topicSize);
	fscanf(fp, "ETA:%lf ", &m_eta); // smoothing beta
	fscanf(fp, "GAMMA:%lf ", &m_gamma); // smoothing alpha
	fscanf(fp, "THETA:%lf ", &m_theta);
	fscanf(fp, "STEP:%d ", &m_maxStep); // EM iteration max step size
	fscanf(fp, "BOUND:%lf ", &m_bound); // EM converge limit
	fclose(fp);

	m_theta_N = log(1.0f-m_theta); // function topic weighting
	m_theta = log(m_theta); // content topic weighting
}

void _strTM::release()
{
	int pointer;

	if ( m_init )
	{
		for (pointer=0; pointer<m_topicSize; pointer++)
		{
			delete[] m_beta[pointer];
			delete[] m_SS_beta[pointer];

			delete[] m_alpha[pointer];
			delete[] m_alpha_G[pointer];
			delete[] m_SS_alpha[pointer];
		}
		delete[] m_beta[pointer];
		delete[] m_SS_beta[pointer];

		delete[] m_alpha;
		delete[] m_alpha_G;
		delete[] m_SS_alpha;
		delete[] m_beta;
		delete[] m_SS_beta;

		delete[] m_pi;
		delete[] m_PPi;
		delete[] m_SS_pi;
	}

	if ( m_loadData && m_init )
	{
		for (pointer=0; pointer<m_maxStnSize; pointer++)
		{
			delete[] m_forward[pointer];
			delete[] m_backward[pointer];
			delete[] m_backpointer[pointer];
		}
		delete[] m_forward;
		delete[] m_backward;
		delete[] m_backpointer;
	}

	if ( m_label2words != NULL )
	{
		for (pointer=0; pointer<(int)m_labelIndex.size(); pointer++)
			delete[] m_label2words[pointer];
		delete[] m_label2words;
		if ( m_topic2label != NULL )
			delete[] m_topic2label;
		delete[] m_labelPrior;
		m_labelIndex.clear();
	}

	cout<<"[Info]model released!"<<endl;
}

bool _strTM::LoadCategory(const char* filename)
{//load document-level labels for each document
	ifstream fileIn(filename);
	if ( fileIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, LoadCategory() method, file "<<filename<<" open failed!"<<endl;
		return false;
	}

	string tmpTxt;
	vector<string> container;
	int docID, labelPtx;
	_doc* tmpDoc;

	while (fileIn.good() && !fileIn.eof())
	{
		getline(fileIn, tmpTxt);
		if ( tmpTxt.empty() )
			continue;

		SplitLine(tmpTxt, " ", container);
		docID = m_corpus.findDoc(container[0]);
		if ( docID == -1 )//no such document in corpus
			continue;
		
		tmpDoc = m_corpus.m_documents[docID];
		tmpDoc->m_labels.assign(container.begin()+1, container.end());//associate it with the labels

		for (labelPtx=1; labelPtx<(int)container.size(); labelPtx++)
		{//scan the document level labels
			if ( m_labelIndex.count(container[labelPtx]) == 0 )
			{//build the label index
				m_labelIndex.insert( make_pair(container[labelPtx], (int)m_labelIndex.size()) );
				m_labelList.push_back(container[labelPtx]);
			}
		}
	}
	fileIn.close();

	init_labelmatrix();
	m_labelSize = (int)m_labelIndex.size();

	cout<<"[Info]Totally load "<<m_labelIndex.size()<<" labels..."<<endl;
	return true;
}

bool _strTM::LoadSentences(const char* filename)
{//load sentences for each document
	ifstream fileIn(filename);
	if ( fileIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, LoadSentences() method, file "<<filename<<" open failed!"<<endl;
		return false;
	}

	_doc* tmpDoc;
	string tmpTxt;
	int docID, docSize = 0;
	bool sof = true;
	while (fileIn.good() && !fileIn.eof())
	{
		getline(fileIn, tmpTxt);
		if ( tmpTxt.empty() )
		{
			sof = true;
			continue;
		}
		else if ( sof == false )
			continue; // escape all those sentences

		docID = m_corpus.findDoc(tmpTxt);
		if ( docID == -1 )//no such document in corpus
		{
			cerr<<"[Warning]No such document "<<tmpTxt<<"!"<<endl;
			sof = false;
			continue;
		}
		else
		{
			tmpDoc = m_corpus.m_documents[docID];
			if ( loadStns(tmpDoc, fileIn) )
				docSize ++;
		}
	}

	cout<<"[Info]Totally load "<<docSize<<" documents with text sentences..."<<endl;
	fileIn.close();

	return true;
}

bool _strTM::loadStns(_doc* Doc, ifstream& fileIn)
{
	string tmpTxt;
	while (fileIn.good() && !fileIn.eof())
	{
		getline(fileIn, tmpTxt);
		if ( tmpTxt.empty() )
			break;
		else
			Doc->m_stnSet.push_back(tmpTxt);
	}

	if ( Doc->m_stnSet.size() != Doc->m_sentences.size() )
	{//inconsistent size of sentences
		Doc->m_stnSet.clear();
		cerr<<"[Warning]In document "<<Doc->m_ID<<" indexed sentences are inconsistent with the original sentence!"<<endl;
		return false;
	}
	return true;
}

void _strTM::init_labelmatrix()
{//build up label to word co-occurrence matrix
	int labelSize = (int)m_labelIndex.size(), labelIndex, labelPtx;
	m_labelPrior = new double[labelSize];
	memset(m_labelPrior, 0, sizeof(double)*labelSize);

	m_label2words = new double*[labelSize];
	for (labelIndex=0; labelIndex<labelSize; labelIndex++)
	{//initialize the label2word matrix
		m_label2words[labelIndex] = new double[m_wordSize];
		memset(m_label2words[labelIndex], 0, sizeof(double)*m_wordSize);		
	}

	//reorder the label indexes for the labels according to the alphabetic order
	labelIndex = 0;
	for (map<string, int>::iterator iter=m_labelIndex.begin(); iter!=m_labelIndex.end(); iter++)
	{
		m_labelList[labelIndex] = iter->first;
		iter->second = labelIndex;
		labelIndex++;
	}

	_doc* tmpDoc;
	for (int docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
	{//associate the words with labels
		tmpDoc = m_corpus.m_documents[docPtx];
		labelSize = (int)tmpDoc->m_labels.size();
		for (labelPtx=0; labelPtx<labelSize; labelPtx++)
		{
			labelIndex = m_labelIndex[tmpDoc->m_labels[labelPtx]];//find the label's index in the label2word matrix
			m_labelPrior[labelIndex]++;
			addwords2matrix(m_label2words[labelIndex], tmpDoc, 1.0/labelSize);//find the word counts
		}		
	}
	
	m_labelSize = (int)m_labelIndex.size();
	L1_Norm(m_labelPrior, m_labelSize, m_gamma);//normalize the label's prior
	for (labelPtx=0; labelPtx<m_labelSize; labelPtx++)
		L1_Norm(m_label2words[labelPtx], m_wordSize, m_eta);// with smoothing
	cout<<"[Info]Label to work matrix initialized!"<<endl;
}

void _strTM::addwords2matrix(double* frequency, const _doc* document, double ratio)
{
	_stn* tmpStn;
	int wordPtx, wordIndex;
	double count;
	for (int stnPtx=0; stnPtx<document->m_docLength;stnPtx++)
	{
		tmpStn = document->m_sentences[stnPtx];
		for (wordPtx=0; wordPtx<tmpStn->m_stnLength; wordPtx++)
		{
			wordIndex = tmpStn->m_words[wordPtx];
			count = tmpStn->m_count[wordPtx];

			frequency[wordIndex] += count * ratio;
		}
	}
}

bool _strTM::LoadData(const char* filename)
{//read the indexed documents
	ifstream fileIn(filename);
	if ( fileIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, LoadData() method, file "<<filename<<" open failed!"<<endl;
		return false;
	}

	string tmpTxt;
	_doc* tmpDoc = NULL;
	_stn* tmpStn = NULL;
	map<int, int> wordindex;
	map<int, int>::iterator worditer;
	vector<string> sentence;
	bool startOfFile = false;
	int wordPtx, word, stnSize = 0, docSize = 0;
	m_avgStnLength = 0;
	while (fileIn.good() && !fileIn.eof())
	{
		getline(fileIn, tmpTxt);
		if ( tmpTxt.empty() || tmpTxt == "\r")
		{//end of a doc
			if ( tmpDoc != NULL )
			{
				if ( tmpDoc->m_docLength == 0 )//empty document, something wrong with the original document
					delete tmpDoc;
				else
				{
					if ( tmpDoc->m_docLength > m_maxStnSize )
						m_maxStnSize = tmpDoc->m_docLength; // get the largest sentence size
					tmpDoc->m_docSize = docSize;
					m_corpus.addDoc(tmpDoc);
				}
				tmpDoc = NULL;
				docSize = 0;
			}
			startOfFile = false;
		}
		else
		{//start of a file
			if (startOfFile == false)
			{
				tmpDoc = new _doc(tmpTxt);
				startOfFile = true;
			}
			else
			{
				SplitLine(tmpTxt, " ", sentence);
				tmpStn = new _stn(tmpDoc->m_docLength);
				stnSize ++;
				
				docSize += (int)sentence.size();//volume of words in this document
				for (wordPtx=0; wordPtx<(int)sentence.size(); wordPtx++)
				{
					word = atoi(sentence[wordPtx].c_str());
					if ( (worditer=wordindex.find(word)) == wordindex.end() )
					{//new word in sentence
						wordindex.insert( make_pair(word, tmpStn->m_stnLength) );
						tmpStn->addWord(word);
					}
					else
						tmpStn->addCount(worditer->second);

					if ( m_wordSize<word )
						m_wordSize = word; // get the size of words, in total of the counts
				}				
				tmpDoc->addStn(tmpStn);
				m_avgStnLength += tmpStn->m_wordSize;
				wordindex.clear();
				tmpStn = NULL;
			}			
		}
	}

	if ( tmpDoc != NULL )
	{//in case the last line is not empty
		if ( tmpDoc->m_docLength > m_maxStnSize )
			m_maxStnSize = tmpDoc->m_docLength; // get the largest sentence size
		m_corpus.addDoc(tmpDoc);
		tmpDoc = NULL;
	}
	m_wordSize++;

	m_avgStnLength /= stnSize;
	cout<<"[Info]Total get "<<m_corpus.m_volume<<" documents, "<<m_wordSize<<" words and "<<stnSize<<" sentences from "<<filename<<", average sentence length is "<<m_avgStnLength<<"..."<<endl;
	
	m_avgStnLength *= exp(m_theta_N);//for dynamically tuning \lambda in each sentence
	m_loadData = true;
	return true;
}

void _strTM::init_model(bool random /* = true */)
{		
	//initialize the model matrices
	m_beta = new double*[m_topicSize+1]; // additional 1 for background topic
	m_SS_beta = new double*[m_topicSize+1];// additional 1 for background topic

	m_alpha = new double*[m_topicSize];// background topic do not have transition
	m_alpha_G = new double*[m_topicSize];// global topic transition matrix
	m_SS_alpha = new double*[m_topicSize];// background topic do not have transition

	m_pi = new double[m_topicSize];//should not start with a background topic
	m_SS_pi = new double[m_topicSize];//should not start with a background topic
	m_PPi = new double[m_topicSize];

	int topicPtx;
	for ( topicPtx=0; topicPtx<m_topicSize; topicPtx++ )
	{
		m_beta[topicPtx] = new double[m_wordSize];
		m_SS_beta[topicPtx] = new double[m_wordSize];

		m_alpha[topicPtx] = new double[m_topicSize];
		m_alpha_G[topicPtx] = new double[m_topicSize];
		m_SS_alpha[topicPtx] = new double[m_topicSize];
	}
	m_beta[topicPtx] = new double[m_wordSize]; // for the additional background topic 
	m_SS_beta[topicPtx] = new double[m_wordSize]; // for the additional background topic 

	//initialize the forward/backward treiliers
	if ( m_loadData )
	{
		m_forward = new double*[m_maxStnSize];
		m_backward = new double*[m_maxStnSize];
		m_backpointer = new int*[m_maxStnSize];
		for (topicPtx=0; topicPtx<m_maxStnSize; topicPtx++)
		{
			m_forward[topicPtx] = new double[m_topicSize];
			m_backward[topicPtx] = new double[m_topicSize];
			m_backpointer[topicPtx] = new int[m_topicSize];
		}
	}
	
	if (random)//random initialization
		rand_model();

	cout<<"[Info]Model initialize with "<<m_topicSize<<" topics!"<<endl;
	m_init = true;
}

void _strTM::rand_model()
{
	int topicPtx, wordPtx, labelSize = 0;

#ifdef SET_PRIOR
	labelSize = (int)m_labelList.size();
#endif//SET_PRIOR

//first initialize the background topic
#ifdef RAND_BACKGROUND
	for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
		m_beta[m_topicSize][wordPtx] = MY_RAND;
	L1_Norm(m_beta[m_topicSize], m_wordSize, 0.1);
	for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
		m_beta[m_topicSize][wordPtx] = log(m_beta[m_topicSize][wordPtx]);
#else//RAND_BACKGROUND
	memset(m_beta[m_topicSize], 0, sizeof(double)*m_wordSize);
	init_background(m_beta[m_topicSize]);//set background topic from the corpus
#endif//RAND_BACKGROUND

	for ( topicPtx=0; topicPtx<m_topicSize; topicPtx++ )
	{
		if ( topicPtx<m_labelSize )
		{//initialize from the data if we have labeled data
			m_pi[topicPtx] = m_labelPrior[topicPtx];
			drawFromPrior(m_beta[topicPtx], m_label2words[topicPtx], 50);//select typical words from prior
		}
		else
		{//randomize
			m_pi[topicPtx] = MY_RAND;
			for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
				m_beta[topicPtx][wordPtx] = MY_RAND;
		}

		for (wordPtx=0; wordPtx<m_topicSize; wordPtx++)
		{
			m_alpha_G[topicPtx][wordPtx] = MY_RAND;
#ifdef SET_PRIOR
			if ( topicPtx == wordPtx )
				m_alpha_G[topicPtx][wordPtx] += m_gamma;//we prefer the self-transition
#endif//SET_PRIOR
		}
	}

#ifdef SET_PRIOR
	getTransitionPrior();
#endif

	L1_Norm(m_pi, m_topicSize, m_gamma);//pi_START is not in the log-space!
	for ( topicPtx=0; topicPtx<m_topicSize; topicPtx++ )
	{
		L1_Norm(m_beta[topicPtx], m_wordSize);
		for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
			m_beta[topicPtx][wordPtx] = log(m_beta[topicPtx][wordPtx]);

		L1_Norm(m_alpha_G[topicPtx], m_topicSize);
	}
}

void _strTM::getTransitionPrior()
{//Using the document level label co-occurrence as prior for the global topic transition probability 
	_doc* doc;
	int labelPtxA, labePtxB, labelSize, labelA, labelB;
	for (int docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
	{
		doc = m_corpus.m_documents[docPtx];
		if ( (labelSize=(int)doc->m_labels.size()) < 2 )
			continue;//no transitions

		for (labelPtxA=0; labelPtxA<labelSize; labelPtxA++)
		{
			labelA = m_labelIndex[doc->m_labels[labelPtxA]];
			for (labePtxB=labelPtxA+1; labePtxB<labelSize; labePtxB++)
			{
				labelB = m_labelIndex[doc->m_labels[labePtxB]];
				m_alpha_G[labelA][labelB] += m_gamma/labelSize;//all the heuristics
				m_alpha_G[labelB][labelA] += m_gamma/labelSize;
			}
		}
	}
}

void _strTM::init_background(double* dist)
{
	for (int docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
		sampleInDoc(dist, m_corpus.m_documents[docPtx]);

	L1_Norm(dist, m_wordSize);
	for (int wordPtx=0; wordPtx<m_wordSize; wordPtx++)
	{
		m_bgTopic.addItem(wordPtx, dist[wordPtx]);// select the topical words from background topic
		dist[wordPtx] = log(dist[wordPtx]);// for the additional background topic 
	}
}

void _strTM::sampleInDoc(double* dist, const _doc* tmpDoc)
{//estimate document language model
	_stn* stn;
	int wordPtx;
	for (int stnPtx=0; stnPtx<tmpDoc->m_docLength; stnPtx++)
	{
		stn = tmpDoc->m_sentences[stnPtx];
		for (wordPtx=0; wordPtx<stn->m_stnLength; wordPtx++)
			dist[stn->m_words[wordPtx]] += stn->m_count[wordPtx];
	}
}

void _strTM::drawFromPrior(double* beta, const double* prior, int size /* = 20 */)
{
	_priQueue<int, double> probList( size<m_wordSize?size:m_wordSize );
	int wordPtx, pos;
	for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
	{
		beta[wordPtx] = MY_RAND/m_wordSize;		
		if ( (pos = m_bgTopic.contains(wordPtx)) == -1 || m_bgTopic.getS(pos) < prior[wordPtx] )//this word is not popular in background
			probList.addItem(wordPtx, prior[wordPtx]);//select the top size words
	}
	
	for(pos=0; pos<size; pos++)
	{
		wordPtx = probList.getT(pos);
		beta[wordPtx] += probList.getS(pos);
	}
}

void _strTM::EM_est()
{
	if ( m_init == false )
		init_model();

	cout<<"[Info]EM procedure start..."<<endl;

	int step = 0, docPtx;
	double pre_likelihood = 1.0f, cur_likehood, converge = 1.0f, likelihood;	

	while ( step<m_maxStep && (step<5 || converge < 0 || converge>m_bound) )
	{
		zero_SS();
		cur_likehood = 0.0f;

		for (docPtx=0; docPtx<m_corpus.m_volume; docPtx++)		
		{
			likelihood = doc_estep(m_corpus.m_documents[docPtx]);
			update_SS(m_corpus.m_documents[docPtx], likelihood);
			cur_likehood += likelihood;
		}
		corpus_mstep();
		cur_likehood /= m_corpus.m_volume;//likelihood per document

		converge = (pre_likelihood-cur_likehood)/pre_likelihood;
		cout<<step<<" "<<cur_likehood<<"@"<<converge<<endl;

		pre_likelihood = cur_likehood;		
		step++;
	}
}

void _strTM::inference(const char* filename)
{
	ofstream resultOut( filename );
	if ( resultOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method inference(), infer "<<filename<<" open failed!"<<endl;
		return;
	}

	int scale = m_corpus.m_volume / 10, *topicDist = new int[m_maxStnSize], docPtx, stnPtx; 
	double likelihood, *problist = new double[m_maxStnSize];
	_doc* Doc = NULL;
	vector<string> container;

	for (docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
	{
		Doc = m_corpus.m_documents[docPtx];
		if ( Doc->m_docLength < 2 )
			continue;//discard too short documents, inconsistent contents

		likelihood = doc_infer(Doc, topicDist);
		resultOut<<Doc->m_ID<<"\t"<<likelihood/Doc->m_docLength;
		for (stnPtx=0; stnPtx<Doc->m_docLength; stnPtx++)
			resultOut<<"\t"<<topicDist[stnPtx];
		resultOut<<endl;

		if ( docPtx % scale == 0 )
			cout<<"Inferring "<<docPtx<<"th document..."<<endl;
	}
	delete[] topicDist;
	delete[] problist;

}

void _strTM::infer4out(const char* filename, double ratio /* = 0.5 */)
{
	ofstream resultOut( filename );
	if ( resultOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method infer4out(), infer result file "<<filename<<" open failed!"<<endl;
		return;
	}

	if (m_topic2label == NULL && m_labelIndex.empty() == false)
		getTopic2LabelMap();

#ifdef HTML_OUTPUT
	resultOut<<"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\"><head><title>Sentence Level Annotation Results</title><link href=\"std.css\" rel=\"stylesheet\" type=\"text/css\"></head><body>"<<endl;
#endif

	int step = 0, scale = m_corpus.m_volume / 10, *topicDist = new int[m_maxStnSize], correct = 0, total = 0, docPtx, counter = 0; 
	double likelihood, *problist = new double[m_maxStnSize];
	_doc* Doc = NULL;

	for (docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
	{
		Doc = m_corpus.m_documents[docPtx];
		if ( Doc->m_docLength < 2 || Doc->m_stnSet.empty() || Doc->m_labels.empty() || MY_RAND < ratio )
			continue;//discard too short documents, inconsistent contents and random show 5% samples

		likelihood = doc_infer(Doc, topicDist, problist);

		//likelihood = greedy_infer(Doc, topicDist, problist);

#ifdef HTML_OUTPUT
		outHTML(resultOut, Doc, topicDist, problist, likelihood);
#else
		outTEXT(resultOut, Doc, topicDist, problist, likelihood);
#endif

		counter++;
		if ( docPtx % scale == 0 )
			cout<<"Inferring "<<docPtx<<"th document..."<<endl;
	}
	delete[] topicDist;
	delete[] problist;

#ifdef HTML_OUTPUT
	resultOut<<"</body></html>"<<endl;
#endif

	resultOut.close();
}

void _strTM::eval(const char* filename){
	_doc* Doc;
	int docPtx, correct = 0, total = 0, acc, *topicDist = new int[m_maxStnSize];
	double *problist = new double[m_maxStnSize];

	int *answer = new int[m_labelSize];
	memset(answer, 0, sizeof(int)*m_labelSize);

	int *guess = new int[m_labelSize];
	memset(guess, 0, sizeof(int)*m_labelSize);

	int **confusionMatrix = new int*[m_labelSize];
	for (docPtx=0; docPtx<m_labelSize; docPtx++)
	{
		confusionMatrix[docPtx] = new int[m_labelSize];
		memset(confusionMatrix[docPtx], 0, sizeof(int) * m_labelSize);
	}

	if ( m_topic2label == NULL && m_labelIndex.empty() == false )
		getTopic2LabelMap();

	for (docPtx=0; docPtx<m_corpus.m_volume; docPtx++)
	{
		Doc = m_corpus.m_documents[docPtx];
		if ( Doc->m_docLength < 2 || Doc->m_stnSet.empty() || Doc->m_labels.empty() )
			continue;//discard too short documents, inconsistent contents and random show 5% samples

		doc_infer(Doc, topicDist, problist);
		acc = ansCheck(Doc, topicDist, answer, guess, confusionMatrix, total);
		correct += acc;
	}

	outPrediction(filename, correct, total, confusionMatrix, answer, guess);

	for (docPtx=0; docPtx<m_labelSize; docPtx++)
		delete[] confusionMatrix[docPtx];
	delete[] answer;
	delete[] guess;
	delete[] confusionMatrix;
	delete[] topicDist;
	delete[] problist;
}

void _strTM::getTopic2LabelMap()
{
	double div, mindiv;
	m_topic2label = new int[m_topicSize];
	int labelPtx, selLabel;
	for (int topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
#ifndef SET_PRIOR
		mindiv = KL_divergency(m_label2words[0], m_beta[topicPtx], m_wordSize);
		selLabel = 0;// scan from the first label

		for (labelPtx=1; labelPtx<(int)m_labelIndex.size(); labelPtx++)
		{
			div = KL_divergency(m_label2words[labelPtx], m_beta[topicPtx], m_wordSize);

			if ( div < mindiv )
			{
				mindiv = div;
				selLabel = labelPtx;
			}
		}
#else
		m_topic2label[topicPtx] = topicPtx;
#endif
	}
}

int _strTM::ansCheck(const _doc* Doc, const int* topicList, int* answer, int* guess, int** confusionMatrix, int& total)
{
	int correct = 0, g, a;
	vector<string> labels;
	for (int stnPtx=0; stnPtx<Doc->m_docLength; stnPtx++)
	{
		SplitLine(Doc->m_stnSet[stnPtx], "\t", labels);
		if ( labels[0] == "NULL" )
			continue;

		a = m_labelIndex[labels[0]];
		g = m_topic2label[topicList[stnPtx]];

		answer[a] ++;
		guess[g] ++;
		confusionMatrix[a][g] ++;
		if (a == g)
			correct++;
		total++;
	}
	return correct;
}

void _strTM::outPrediction(const char* filename, int correct, int total, int** confusionMatrix, int* answer, int* guess)
{
	ofstream tmpOut(filename);
	if ( tmpOut.fail() )
	{
		cerr<<"[Error]In _procModel.cpp, outPrediction() method, file "<<filename<<" open failed!"<<endl;
		return;
	}

	double recall = 0, precision = 0, r, p;
	int step, docPtx;
	for (step=0; step<m_labelSize; step++)
		tmpOut<<"\t"<<m_labelList[step];
	tmpOut<<"\tRecall\tPrecision"<<endl;
	for (docPtx=0; docPtx<m_labelSize; docPtx++)
	{
		for (step=0; step<m_labelSize; step++)
		{
			if (step==0)
				tmpOut<<m_labelList[docPtx]<<"\t";
			tmpOut<<confusionMatrix[docPtx][step]<<"\t";
		}
		r = (answer[docPtx]==0?0:(double)confusionMatrix[docPtx][docPtx]/answer[docPtx]); //recall
		p = (guess[docPtx]==0?0:(double)confusionMatrix[docPtx][docPtx]/guess[docPtx]); //precision
		tmpOut<<r<<"\t"<<p<<endl;
		recall += r;
		precision += p;
	}
	cout<<"[Info]Accuracy "<<(double)correct/total<<" Recall: "<<recall/m_labelSize<<" Precision: "<<precision/m_labelSize<<endl;
	tmpOut<<(double)correct/total<<"\t"<<recall/m_labelSize<<"\t"<<precision/m_labelSize<<endl;
	tmpOut.close();
}

void _strTM::zero_SS()
{//clear the sufficient statistics
	memset(m_SS_pi, 0, m_topicSize*sizeof(double));
	for (int topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
		memset(m_SS_alpha[topicPtx], 0, m_topicSize*sizeof(double));
		memset(m_SS_beta[topicPtx], 0, m_wordSize*sizeof(double));
	}
#ifdef UPDATE_BACKGROUND
	memset(m_SS_beta[m_topicSize], 0, m_wordSize*sizeof(double));
#endif
}

double _strTM::topic2word(int topicPtx, int wordPtx)
{//p(w|z) = \lambda * \beta_{zn} + (1-\lambda) * \beta_{z_B n}
	return log_sum(m_theta + m_beta[topicPtx][wordPtx], m_theta_N + m_beta[m_topicSize][wordPtx]);
}

double _strTM::topic2stn(int topicPtx, const _stn* stn)
{//p(s|z) = \prod_{w\in s} p(w|z)
	double prob = 0, theta = m_theta, theta_N = m_theta_N;

	//dynamically tune the lambda weight ???
	m_theta = stn->m_wordSize / (m_avgStnLength + stn->m_wordSize);//document length normalization
	m_theta_N = log(1.0-m_theta);
	m_theta = log(m_theta);

	for (int wordPtx=0; wordPtx<stn->m_stnLength; wordPtx++)
		prob += stn->m_count[wordPtx] * topic2word(topicPtx, stn->m_words[wordPtx]);

	//restore the global setting
	m_theta = theta;
	m_theta_N = theta_N;
	return prob;
}

void _strTM::getPrior(const vector<string>& labels)
{// document specific topic initial distribution
	int index, tagA, tagB;
	memcpy(m_PPi, m_pi, sizeof(double)*m_topicSize);//global initial status
	for (index=0; index<m_topicSize; index++)
		memcpy(m_alpha[index], m_alpha_G[index], sizeof(double)*m_topicSize);	

#ifdef SET_PRIOR
	int pos, size = (int)labels.size(); // draw from local prior
	double guess = 1.0/size; //default transition

	for (index=0; index<size; index++)
	{//draw from the document-level tags
		if ( labels[index] == "NULL" )
			continue;

		tagA = m_labelIndex[labels[index]];
		m_PPi[tagA] += guess; // just a heuristic setting for initial probability
		m_alpha[tagA][tagA] += guess;
		for (pos=index+1; pos<size; pos++)
		{//heuristic for local transition probability
			tagB = m_labelIndex[labels[pos]];
			m_alpha[tagA][tagB] += guess;
			m_alpha[tagB][tagA] += guess;
		}
	}	
#endif

	L1_Norm(m_PPi, m_topicSize);
	for (tagA=0; tagA<m_topicSize; tagA++)
	{//normalize
		m_PPi[tagA] = log(m_PPi[tagA]);

		L1_Norm(m_alpha[tagA], m_topicSize);
		for (tagB=0; tagB<m_topicSize; tagB++)
			m_alpha[tagA][tagB] = log(m_alpha[tagA][tagB]);
	}
}

//void _strTM::getPrior(const vector<string>& labels)
//{// document specific topic initial distribution
//	int index, tagA, tagB;
//	memcpy(m_PPi, m_pi, sizeof(double)*m_topicSize);//global initial status
//
//#ifdef SET_PRIOR
//	int pos, size = (int)labels.size(); // draw from global prior
//	double init = 1e-2; //default transition
//
//	if (size > 0)
//	{//with document-level labels
//		for (tagA=0; tagA<m_topicSize; tagA++)
//		{//initialize local transition matrix by global transition matrix \alpha_{G}
//			m_alpha[tagA][tagA] = init;
//			if ( tagA>=m_labelSize )//common topical components: Part I
//				m_alpha[tagA][tagA] += m_alpha_G[tagA][tagA];
//
//			for (tagB=tagA+1; tagB<m_topicSize; tagB++){
//				m_alpha[tagA][tagB] = init;
//				m_alpha[tagB][tagA] = init;
//
//				if(tagA>=m_labelSize && tagB>=m_labelSize)
//				{//common topical components: Part I
//					m_alpha[tagA][tagB] += m_alpha_G[tagA][tagB];
//					m_alpha[tagB][tagA] += m_alpha_G[tagB][tagA];
//				}
//			}
//		}
//	}
//	else
//	{//in case for the unlabeled data
//		for (index=0; index<m_topicSize; index++)
//			memcpy(m_alpha[index], m_alpha_G[index], sizeof(double)*m_topicSize);
//	}
//
//	for (index=0; index<size; index++)
//	{//draw from the document-level tags
//		if ( labels[index] == "NULL" )
//			continue;
//
//		tagA = m_labelIndex[labels[index]];
//		m_PPi[tagA] += 1.0; // just a heuristic setting for initial probability
//
//		m_alpha[tagA][tagA] += m_alpha_G[tagA][tagA];
//		for (pos=index+1; pos<size; pos++)
//		{//heuristic for local transition probability
//			tagB = m_labelIndex[labels[pos]];
//			m_alpha[tagA][tagB] += m_alpha_G[tagA][tagB];
//			m_alpha[tagB][tagA] += m_alpha_G[tagB][tagA];
//		}
//
//		for (tagB=m_labelSize; tagB<m_topicSize; tagB++)
//		{//common topical components: Part II
//			m_alpha[tagA][tagB] += m_alpha_G[tagA][tagB];
//			m_alpha[tagB][tagA] += m_alpha_G[tagB][tagA];
//		}
//	}	
//#else
//	for (index=0; index<m_topicSize; index++)
//		memcpy(m_alpha[index], m_alpha_G[index], sizeof(double)*m_topicSize);
//#endif
//
//	L1_Norm(m_PPi, m_topicSize);
//	for (tagA=0; tagA<m_topicSize; tagA++)
//	{//normalize
//		m_PPi[tagA] = log(m_PPi[tagA]);
//
//		L1_Norm(m_alpha[tagA], m_topicSize);
//		for (tagB=0; tagB<m_topicSize; tagB++)
//			m_alpha[tagA][tagB] = log(m_alpha[tagA][tagB]);
//	}
//}

double _strTM::doc_estep(const _doc* document)
{
	int topicPtx, stnLength = document->m_docLength, stnPtx;
	double likelihood = 0.0f;
	_stn *forwardStn = document->m_sentences[0], *backwardStn;

	getPrior(document->m_labels);//if we have document level labels in advance

	//init forward/backward treillers
	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
		m_forward[0][topicPtx] = m_PPi[topicPtx] + topic2stn(topicPtx, forwardStn);
		m_backward[stnLength-1][topicPtx] = 0;
	}

	//forward/backward procedure
	for (stnPtx=1; stnPtx<stnLength; stnPtx++)
	{//for the topic to sentence probability, should we take the function part into consideration?
		forwardStn = document->m_sentences[stnPtx];
		backwardStn = document->m_sentences[stnLength-stnPtx];

		for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
		{
			m_forward[stnPtx][topicPtx] = forwardProb(topicPtx, stnPtx-1) + topic2stn(topicPtx, forwardStn);
			m_backward[stnLength-stnPtx-1][topicPtx] = backwardProb(topicPtx, stnLength-stnPtx, backwardStn);
#ifdef _DEBUG
			if (m_forward[stnPtx][topicPtx] < -MY_INFINITE || m_backward[stnLength-stnPtx-1][topicPtx] < -MY_INFINITE) 
				cerr<<"[Warning]In "<<document->m_ID<<" calculation underflow!"<<endl;
#endif//_DEBUG
		}
	}

	stnPtx = stnLength-1;
	likelihood = m_forward[stnPtx][0];
	for (topicPtx=1; topicPtx<m_topicSize; topicPtx++)
		likelihood = log_sum(likelihood, m_forward[stnPtx][topicPtx]);

	return likelihood;
}

double _strTM::greedy_infer(const _doc* document, int* topicDist, double* problist)
{//only searching for local optimal in each state
	int topicPtx, stnLength = document->m_docLength, stnPtx;
	double likelihood = 0.0f, maxProb, prob;
	_stn* tmpStn = NULL;

	//greedy procedure
	for (stnPtx=0; stnPtx<stnLength; stnPtx++)
	{
		tmpStn = document->m_sentences[stnPtx];
		maxProb = topic2stn(0, tmpStn);
		topicDist[stnPtx] = 0;
		for (topicPtx=1; topicPtx<m_topicSize; topicPtx++)
		{
			prob = topic2stn(topicPtx, tmpStn);
			if( prob > maxProb )
			{
				maxProb = prob;
				topicDist[stnPtx] = topicPtx;
			}
		}	
		likelihood += maxProb;
		problist[stnPtx] = maxProb;
	}	

	return likelihood;
}

double _strTM::doc_infer(const _doc* document, int* topicDist, double* problist/* =NULL */)
{//viterbi inference
	int topicPtx, stnLength = document->m_docLength, stnPtx, selTopic;
	double likelihood = 0.0f, maxProb;
	_stn* tmpStn = document->m_sentences[0];

	getPrior(document->m_labels);

	//init forward/backward treillers
	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
		m_forward[0][topicPtx] = m_PPi[topicPtx] + topic2stn(topicPtx, tmpStn);
		m_backpointer[0][topicPtx] = -1; // start point
	}

	//viterbi procedure
	for (stnPtx=1; stnPtx<stnLength; stnPtx++)
	{
		tmpStn = document->m_sentences[stnPtx];
		for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
		{
			m_forward[stnPtx][topicPtx] = viterbiProb(topicPtx, stnPtx-1, selTopic) + topic2stn(topicPtx, tmpStn);
			m_backpointer[stnPtx][topicPtx] = selTopic;
		}		
	}

	stnPtx = stnLength-1;//the last sentence of the document	
	likelihood = m_forward[stnPtx][0];
	selTopic = 0;
	for (topicPtx=1; topicPtx<m_topicSize; topicPtx++)
	{
		maxProb = m_forward[stnPtx][topicPtx];
		if ( likelihood < maxProb )
		{
			likelihood = maxProb;
			selTopic = topicPtx;
		}
	}		

	vector<double> ranker(m_topicSize);
	for (stnPtx=stnLength-1; stnPtx>=0; stnPtx--)
	{
		topicDist[stnPtx] = selTopic;

		if (problist)
		{
 			ranker.assign(m_forward[stnPtx], m_forward[stnPtx]+m_topicSize);
 			make_heap(ranker.begin(), ranker.end());
 			problist[stnPtx] = exp(m_forward[stnPtx][selTopic] - ranker.front());
		}
 
		selTopic = m_backpointer[stnPtx][selTopic];
	}

	return likelihood;
}

double _strTM::forwardProb(int topicPtx, int preStnPos)
{
	double prob = m_forward[preStnPos][0] + m_alpha[0][topicPtx];
	for (int preTopic=1; preTopic<m_topicSize; preTopic++)
		prob = log_sum(prob, m_forward[preStnPos][preTopic] + m_alpha[preTopic][topicPtx]);
	return prob;
}

double _strTM::viterbiProb(int topicPtx, int preStnPos, int& selTopic)
{
	double maxProb = m_forward[preStnPos][0] + m_alpha[0][topicPtx], prob;
	selTopic = 0;
	for (int preTopic=1; preTopic<m_topicSize; preTopic++)
	{
		prob = m_forward[preStnPos][preTopic] + m_alpha[preTopic][topicPtx];
		if ( prob > maxProb )
		{
			maxProb = prob;
			selTopic = preTopic;
		}
	}
	return maxProb;
}

double _strTM::backwardProb(int topicPtx, int nextStnPos, const _stn *nextStn)
{
	double prob = m_alpha[topicPtx][0] + topic2stn(0, nextStn) + m_backward[nextStnPos][0];
	for (int nextTopic=1; nextTopic<m_topicSize; nextTopic++)
		prob = log_sum(prob, m_alpha[topicPtx][nextTopic] + topic2stn(nextTopic, nextStn) + m_backward[nextStnPos][nextTopic]);
	return prob;
}

void _strTM::update_SS(const _doc* document, double likelihood)
{
	if ( likelihood < -MY_INFINITE )
		cerr<<"[Error]likelihood calculation underflow!"<<endl;

	int topicPtx, nextTopic, stnLength = document->m_docLength;
	double prob, ratio = (m_labelSize>0 && document->m_labels.empty()?0.01:1);
	for (int stnPtx=0; stnPtx<stnLength; stnPtx++)
	{
		for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
		{
			prob = ratio * exp(m_forward[stnPtx][topicPtx] + m_backward[stnPtx][topicPtx] - likelihood);
			update_SS_beta(document->m_sentences[stnPtx], topicPtx, prob);

			if ( stnPtx == 0 )
				m_SS_pi[topicPtx] += prob;
		}

		if ( stnPtx == stnLength-1 )
			break;

		for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)	
		{
			for (nextTopic=0; nextTopic<m_topicSize; nextTopic++)
			{
				prob = m_forward[stnPtx][topicPtx] + m_alpha[topicPtx][nextTopic] + topic2stn(nextTopic, document->m_sentences[stnPtx+1]) 
					+ m_backward[stnPtx+1][nextTopic] - likelihood;

				m_SS_alpha[topicPtx][nextTopic] += ratio * exp(prob);
			}
		}
	}
}

void _strTM::update_SS_beta(const _stn* sentence, int topicPtx, double prob)
{//how to access each word's contribution
	//all the words share the same p(s,z|d) for this sentence?
	int word;
	double conProb, total, count;
	double theta = m_theta, theta_N = m_theta_N;

	//dynamically tune the lambda weight
	m_theta = sentence->m_wordSize / (m_avgStnLength + sentence->m_wordSize);//document length normalization
	m_theta_N = log(1.0 - m_theta);
	m_theta = log(m_theta);

	for (int wordPtx=0; wordPtx<sentence->m_stnLength; wordPtx++)
	{
		word = sentence->m_words[wordPtx];
		count = prob * sentence->m_count[wordPtx];
		total = log_sum(m_theta + m_beta[topicPtx][word], m_theta_N + m_beta[m_topicSize][word]);

 		conProb = exp(m_theta + m_beta[topicPtx][word] - total);		
		m_SS_beta[topicPtx][word] += count * conProb;//how to update p(w|z), evenly distribute is not reasonable!

#ifdef UPDATE_BACKGROUND		
		m_SS_beta[m_topicSize][word] += count * (1-conProb);//how to update p(w|z), evenly distribute is not reasonable!		
#endif
	}

	//restore the global setting
	m_theta = theta;
	m_theta_N = theta_N;
}

void _strTM::corpus_mstep()
{
	int topicPtx, wordPtx;
	double sum, dirSum, topicSum = 0, piSum = 0;
	double etaSum = m_wordSize*m_eta, gammaSum = m_topicSize*m_gamma;
	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
		//update the emission matrix: beta
		sum = m_SS_beta[topicPtx][0];
		for (wordPtx=1; wordPtx<m_wordSize; wordPtx++)
			sum += m_SS_beta[topicPtx][wordPtx];
		dirSum = Digamma(sum + etaSum);//Dirichlet smoothing

		for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
			m_beta[topicPtx][wordPtx] = Digamma(m_SS_beta[topicPtx][wordPtx] + m_eta) - dirSum;

		//update the topic initial/ending distribution: pi_START, pi_END
		piSum += m_SS_pi[topicPtx];

		//update the transition matrix: alpha
		sum = m_SS_alpha[topicPtx][0]; 
		for (wordPtx=1; wordPtx<m_topicSize; wordPtx++)
			sum += m_SS_alpha[topicPtx][wordPtx];
		dirSum = Digamma(sum + gammaSum);//Dirichlet smoothing

		for (wordPtx=0; wordPtx<m_topicSize; wordPtx++)		
			m_alpha_G[topicPtx][wordPtx] = exp(Digamma(m_SS_alpha[topicPtx][wordPtx] + m_gamma) - dirSum);//alpha_G is not in the log space!
	}

	piSum = Digamma(piSum + gammaSum);//Dirichlet smoothing
	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++){
		m_pi[topicPtx] = exp(Digamma(m_SS_pi[topicPtx]) - piSum);//pi_START is not in the log space!
	}

#ifdef UPDATE_BACKGROUND
	sum = m_SS_beta[m_topicSize][0];
	for (wordPtx=1; wordPtx<m_wordSize; wordPtx++)
		sum += m_SS_beta[m_topicSize][wordPtx];
	dirSum = Digamma(sum + etaSum);//Dirichlet smoothing

	for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
		m_beta[m_topicSize][wordPtx] = Digamma(m_SS_beta[m_topicSize][wordPtx] + m_eta) - dirSum;
#endif//UPDATE_BACKGROUND
}

void _strTM::SaveModel(const char* model_root, const char* seqID/* =NULL */)
{
	string modelDir(model_root);

	ofstream modelOut( ((modelDir)+"/model.beta" + seqID).c_str() );
	if ( modelOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/model.beta open failed!"<<endl;
		return;
	}
	
	//save beta
	int topicPtx, wordPtx;
	for (topicPtx=0; topicPtx<=m_topicSize; topicPtx++)
	{
		for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
			modelOut<<m_beta[topicPtx][wordPtx]<<" ";
		modelOut<<endl;
	}
	modelOut.close();
	cout<<"[Info]beta matrix saved!"<<endl;

	//save alpha
	modelOut.open( ((modelDir)+"/model.alpha" + seqID).c_str() );
	if ( modelOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/model.alpha open failed!"<<endl;
		return;
	}

	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
	{
		for (wordPtx=0; wordPtx<m_topicSize; wordPtx++)
			modelOut<<m_alpha_G[topicPtx][wordPtx]<<" ";
		modelOut<<endl;
	}
	modelOut.close();
	cout<<"[Info]alpha matrix saved!"<<endl;

	//save pi
	modelOut.open( ((modelDir)+"/model.pi" + seqID).c_str() );
	if ( modelOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/model.pi open failed!"<<endl;
		return;
	}

	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)	
		modelOut<<m_pi[topicPtx]<<" ";
	modelOut.close();
	cout<<"[Info]pi matrix saved!"<<endl;

	//save other
	modelOut.open( ((modelDir)+"/model.other").c_str() );
	if ( modelOut.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/model.other open failed!"<<endl;
		return;
	}
	
	modelOut<<m_topicSize<<"\t"<<m_wordSize<<"\t"<<m_eta<<"\t"<<m_gamma<<"\t"<<exp(m_theta);
	modelOut.close();


 	//if ( m_labelList.empty() == false )
 	//{//save labels
 	//	//save word distribution under labels
 	//	modelOut.open( ((modelDir)+"/label.name").c_str() );
 	//	if ( modelOut.fail() )
 	//	{
 	//		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/label.name open failed!"<<endl;
 	//		return;
 	//	}
 	//	
 	//	for (topicPtx=0; topicPtx<(int)m_labelList.size(); topicPtx++)
 	//		modelOut<<m_labelList[topicPtx]<<" "<<m_labelPrior[topicPtx]<<endl;
 	//	modelOut.close();
 	//	cout<<"[Info]Label names saved!"<<endl;
 
 	//	//save word distribution under labels
 	//	modelOut.open( ((modelDir)+"/label.word").c_str() );
 	//	if ( modelOut.fail() )
 	//	{
 	//		cerr<<"[Error]In _Model.cpp, method SaveModel(), file "<<model_root<<"/label.word open failed!"<<endl;
 	//		return;
 	//	}
 
 	//	for (topicPtx=0; topicPtx<(int)m_labelList.size(); topicPtx++)
 	//	{
 	//		for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
 	//			modelOut<<log(m_label2words[topicPtx][wordPtx])<<" ";
 	//		modelOut<<endl;
 	//	}
 	//	modelOut.close();
 	//	cout<<"[Info]Label words saved!"<<endl;
 	//}
	cout<<"[Info]other file saved!"<<endl;
}

void _strTM::LoadModel(const char* model_root)
{
	string modelDir(model_root);
	int topicPtx, wordPtx;

	ifstream modelIn( ((modelDir)+"/model.other").c_str() );
	if ( modelIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method LoadModel(), file "<<model_root<<"/model.ohter open failed!"<<endl;
		return;
	}

	//load other file first, for the statistics of the model	
	modelIn>>m_topicSize;
	modelIn>>m_wordSize;
	modelIn>>m_eta;
	modelIn>>m_gamma;	
	modelIn.close();

	if ( m_init )
		release();
	init_model(false);

	//load beta
	ifstream betaIn( ((modelDir)+"/model.beta").c_str(), ios::in );
	if ( betaIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method LoadModel(), file "<<model_root<<"/model.beta open failed!"<<endl;
		return;
	}	
	
	for (topicPtx=0; topicPtx<=m_topicSize; topicPtx++)
		for (wordPtx=0; wordPtx<m_wordSize; wordPtx++)
			betaIn>>m_beta[topicPtx][wordPtx];
	betaIn.close();
	cout<<"[Info]beta matrix loaded!"<<endl;

	//load alpha
	ifstream alphaIn( ((modelDir)+"/model.alpha").c_str() );
	if ( alphaIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method LoadModel(), file "<<model_root<<"/model.alpha open failed!"<<endl;
		return;
	}

	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)
		for (wordPtx=0; wordPtx<m_topicSize; wordPtx++)
			alphaIn>>m_alpha_G[topicPtx][wordPtx];
	alphaIn.close();
	cout<<"[Info]alpha matrix loaded!"<<endl;

	//load pi
	ifstream piIn( ((modelDir)+"/model.pi").c_str() );
	if ( piIn.fail() )
	{
		cerr<<"[Error]In _Model.cpp, method LoadModel(), file "<<model_root<<"/model.pi open failed!"<<endl;
		return;
	}

	for (topicPtx=0; topicPtx<m_topicSize; topicPtx++)	
		piIn>>m_pi[topicPtx];
	piIn.close();
	cout<<"[Info]pi matrix loaded!"<<endl;
}

void _strTM::outTEXT(ofstream& resultOut, _doc* Doc, const int* topicDist, const double* problist, double likelihood)
{
	int stnPtx, labelPtx;
	resultOut<<likelihood/Doc->m_docLength<<"\t"<<Doc->m_ID<<"\t";
	//the overall labels
	for (stnPtx=0; stnPtx<(int)Doc->m_labels.size(); stnPtx++)
		resultOut<<Doc->m_labels[stnPtx]<<"\t";
	resultOut<<endl;

	for (stnPtx=0; stnPtx<Doc->m_docLength; stnPtx++)
	{
		if ( m_labelIndex.empty() == false && topicDist[stnPtx] < m_labelSize )
		{
			labelPtx = m_topic2label[topicDist[stnPtx]];
			resultOut<<"\t"<<m_labelList[labelPtx]<<"\t\t";
		}
		else
			resultOut<<"\tNULL:"<<topicDist[stnPtx]<<"\t";
		resultOut<<Doc->m_stnSet[stnPtx]<<endl;
	}
	resultOut<<endl;
}

void _strTM::outHTML(ofstream& resultOut, _doc* Doc, const int* topicDist, const double* problist, double likelihood)
{
	int stnPtx, labelPtx;

	resultOut<<"<p><font size=\"5\">"<<Doc->m_ID<<"&nbsp; &nbsp;"<<likelihood<<"</font>&nbsp; &nbsp;";
	char color[7] = {0, };
	resultOut<<"<br /><table border=\"0\">"<<endl;

	for (stnPtx=0; stnPtx<Doc->m_docLength; stnPtx++)
	{		
		labelPtx = topicDist[stnPtx];
		GET_COLOR(color, labelPtx);
		resultOut<<"<tr><td valign=\"top\"><font color=\"#"<<color<<"\" size=\"4\">"<<labelPtx<<"</font></td>"
			<<"<td valign=\"top\"><font color=\"#000000\" size=\"3\">"<<problist[stnPtx]<<"</font></td>"
			<<"<td><font color=\"#"<<color<<"\" size = \"3\">"<<Doc->m_stnSet[stnPtx]<<"</font></td></tr>"<<endl;
		// 			resultOut<<"<font color=\"#"<<color<<"\" size = \"3\">"<<Doc->m_stnSet[stnPtx]<<"</font> ";
	}
	resultOut<<"</table></p>"<<endl;
}