#include <vector>
#include <cmath>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <map>
#include <ctime>
#include <list>
#include "corpus.h"
#include "MyUtil.h"
#include "priority_queue.h"
using namespace std;

#define RAND_BACKGROUND
//#define UPDATE_BACKGROUND
#define HTML_OUTPUT
#define SET_PRIOR
#define GET_COLOR(s, i)	(sprintf(s, "%02x%02x%02x", (3*i+1024) * 4 % 255, (3*i+1024) * 2 % 255, (3*i+1024) * 3 % 255))
#define _AD_DATA_

typedef pair<int, double> EDGE;

class _strTM
{
public:
	
#ifdef _TRACE_OUT_
	_strTM(const char* filename) : m_bgTopic(20), m_tmpOut("../Data/trace.dat", ios::app)
#else
	_strTM(const char* filename) : m_bgTopic(20)
#endif
	{
		LoadSettings(filename);

		m_maxStnSize = -1;
		m_wordSize = -1;
		m_init = false;
		m_loadData = false;
		srand((unsigned int)time(NULL));

		m_label2words = NULL;
		m_topic2label = NULL;
		m_labelSize = 0;//a default setting
	}

	~_strTM()
	{
		release();
#ifdef _TRACE_OUT_
		m_tmpOut.close();
#endif
	}

	void SaveModel(const char* model_root, const char* seqID="");

	void LoadModel(const char* model_root);

	bool LoadData(const char* filename);

	bool LoadCategory(const char* filename);

	bool LoadSentences(const char* filename);

	void inference(const char* filename);

	void infer4out(const char* filename, double ratio = 0.5);

	void eval(const char* filename);

	void EM_est();

public:
	void LoadSettings(const char* filename);

	bool loadStns(_doc* Doc, ifstream& fileIn);

	void release();

	void init_model(bool random = true);

	void rand_model();

	void zero_SS();

	void init_background(double* dist);

	void update_SS(const _doc* document, double likelihood);

	void update_SS_beta(const _stn* sentence, int topicPtx, double prob);

	void corpus_mstep();

	double forwardProb(int topicPtx, int stnPos);

	double viterbiProb(int topicPtx, int stnPos, int& selTopic);

	double backwardProb(int topicPtx, int stnPos, const _stn* nextStn);

	int ansCheck(const _doc* Doc, const int* topicList, int* answer, int* guess, int** confusionMatrix, int& total);

	void outPrediction(const char* filename, int correct, int total, int** confusionMatrix, int* answer, int* guess);

	void outHTML(ofstream& resultOut, _doc* Doc, const int* topicDist, const double* problist, double likelihood);

	void outTEXT(ofstream& resultOut, _doc* Doc, const int* topicDist, const double* problist, double likelihood);

	void getTopic2LabelMap();

protected:
	void sampleInDoc(double* dist, const _doc* tmpDoc);

	void init_labelmatrix();

	void addwords2matrix(double* frequency, const _doc* document, double ratio);

	void getPrior(const vector<string>& labels);

	double doc_estep(const _doc* document);

	double doc_infer(const _doc* document, int* topicDist, double* problist=NULL);

	double greedy_infer(const _doc* document, int* topicDist, double* problist);

	double topic2stn(int topicPtx, const _stn* stn);

	double topic2word(int topicPtx, int wordPtx);

	void drawFromPrior(double* beta, const double* prior, int size = 20);

	void getTransitionPrior();//a tentative approach for transition prior modeling

private:
	double** m_SS_beta; // sufficient statistic for beta
	double** m_SS_alpha; // sufficient statistic for alpha
	double* m_SS_pi; // sufficient statistic for pi_START

protected:
	int m_topicSize; // topic size
	int m_wordSize; // word size
	int m_maxStnSize; // maximal sentence size
	int m_maxStep; // max EM iteration step
	double m_bound; // EM converge limit 

	//hyper parameters
	double m_eta; // smoothing factor of beta (Dirichlet prior)
	double m_gamma; // smoothing factor of alpha (Dirichlet prior)
	double m_theta; // currently global mixture weighting for content topic and function topics
	double m_theta_N; // log(1-\theta)

	//model parameters
	double** m_beta; // topic to word matrix
	double** m_alpha_G; // topic transition matrix (global)
	double** m_alpha; // topic transition matrix (local)
	double* m_pi; // topic initial distribution
	double* m_PPi; // add strong prior to it

	//HMM working set
	double** m_forward; // forward probability
	double** m_backward; // backward probability
	int** m_backpointer; // for viterbi algorithm

	_corpus m_corpus; // corpus
	bool m_init; // indicator
	bool m_loadData; 

	double** m_label2words; // simple frequency matrix for terms under label
	double* m_labelPrior; // statistics for labels
	int* m_topic2label; // mapping from topics to labels
	map<string, int> m_labelIndex; // look-up label by index ID
	vector<string> m_labelList; // list of labels
	int m_labelSize;

	_priQueue<int, double> m_bgTopic;

/////temporary variables
#ifdef _TRACE_OUT_
	ofstream m_tmpOut;
#endif

	double m_avgStnLength;
};
