#include <vector>
#include <string>
#include <map>
#pragma warning(disable : 4996)
using namespace std;

#define MY_INFINITE 1e10
#define MY_RAND ((double)(0.1+rand())/((double)(1.0f+RAND_MAX)))

struct _stn
{
	int m_ID;
	int m_stnLength;
	int m_wordSize;
	vector<int> m_words;
	vector<int> m_count;

	_stn(int ID)
	{
		m_ID = ID;
		m_stnLength = 0;
		m_wordSize = 0;
	}

	~_stn()
	{
		m_words.clear();
		m_count.clear();
	}

	void addWord(int word)
	{
		m_words.push_back(word);
		m_count.push_back(1);
		m_wordSize++;
		m_stnLength++;
	}

	void addCount(int pos)
	{
		m_count[pos]++;
		m_wordSize++;
	}
};

struct _doc
{
	string m_ID;
	int m_docLength;
	int m_docSize;
	vector<_stn*> m_sentences;
	vector<string> m_labels;
	vector<string> m_stnSet; // to store the original sentences

	_doc(const string& id)
	{
		m_ID = id;
		m_docLength = 0;
		m_docSize = 0;
	}

	~_doc()
	{
		for (int stnPtx=0; stnPtx<(int)m_sentences.size(); stnPtx++)
			delete m_sentences[stnPtx];
		m_sentences.clear();
	}

	void addStn(_stn*& stn)
	{
		m_sentences.push_back(stn);
		m_docLength++;
	}

	void copy(_doc* tmpDoc)
	{
		tmpDoc->m_docLength = m_docLength;
		tmpDoc->m_docSize = m_docSize;
		tmpDoc->m_labels.clear();
		for(int i=0; i<(int)m_labels.size(); i++)
			tmpDoc->m_labels.push_back(m_labels[i]);

		tmpDoc->m_sentences.clear();
		for(int i=0; i<m_docLength; i++)
			tmpDoc->m_sentences.push_back(m_sentences[i]);
	}

	bool hasLabel(const string& label)
	{
		for (int i=0; i<(int)m_labels.size(); i++)
			if ( label == m_labels[i] )
				return true;
		return false;
	}

	void permutate(){
		int j;
		_stn* tmpStn = NULL;
		for(int i=0; i<m_docLength; i++)
		{
			j = (int)(MY_RAND * m_docLength) % m_docLength;			
			tmpStn = m_sentences[i];
			m_sentences[i] = m_sentences[j];
			m_sentences[j] = tmpStn;
		}
	}

	void getStnIndex(vector<int>& indexes){
		indexes.clear();
		for(int i=0; i<m_docLength; i++)
			indexes.push_back(m_sentences[i]->m_ID);
	}
};

struct _corpus
{
	int m_volume;
	map<string, int> m_docIndex; // to look up documents by doc ID
	vector<_doc*> m_documents;

	_corpus()
	{
		m_volume = 0;
	}

	~_corpus()
	{
		for (int docPtx=0; docPtx<m_volume; docPtx++)
			delete m_documents[docPtx];
		m_documents.clear();
	}

	void addDoc(_doc*& doc)
	{
		m_volume++;
		m_docIndex.insert( make_pair(doc->m_ID, (int)m_documents.size()) );
		m_documents.push_back(doc);
	}

	int findDoc(const string& title)
	{
		if ( m_docIndex.count(title) != 0 )
			return m_docIndex[title];
		else
			return -1;
	}
};
