#pragma once
#include <vector>
#include <string>
#include <fstream>
#include <set>
using namespace std;

template <class _T, class _S> class _priQueue
{
public:
	_priQueue(int size){
		m_size = size;
	};

	~_priQueue(){
		m_storage.clear();
		m_score.clear();
	};

	void addItem(const _T& item, const _S& score, bool unique = false)
	{
		if ( m_score.empty() )
		{
			m_score.push_back(score);
			m_storage.push_back(item);
			return;
		}

		if (unique && m_keySet.insert(item).second == false)
		{//duplicated keys
			return;
		}

		int pos = search(score);
		if ( (int)m_score.size() < m_size )
		{
			m_score.insert(m_score.begin()+pos, score);
			m_storage.insert(m_storage.begin()+pos, item);
		}
		else if ( pos < m_size )
		{
			m_score.insert(m_score.begin()+pos, score);
			m_storage.insert(m_storage.begin()+pos, item);

			m_score.pop_back();
			m_storage.pop_back();
		}
	}

	void outItem(ofstream& fileOut){
		for (int i=0; i<(int)m_score.size(); i++)
			fileOut<<"\t"<<m_storage[i]<<"\t"<<m_score[i]<<endl;
	}

	_T getT(int pos){return m_storage[pos];}

	_S getS(int pos){return m_score[pos];}

	void clear(bool cSize = false){
		if ( cSize )
			m_size = 0;
		m_storage.clear();
		m_score.clear();
	}

	int getSize(){return (int)m_storage.size();}
	void setSize(int size){m_size=size;}

	int contains(const _T& key){
		int pos = 0;
		for (pos=0; pos<(int)m_storage.size(); pos++)
			if (m_storage[pos] == key)
				return pos;
		return -1;
	}

private:
	int search(const _S& score){
		return binarySearch(0, (int)m_score.size(), score);
	}

	int binarySearch(int left, int right, const _S& score)
	{
		if ( score >= m_score[left] )
			return left;
		else if ( score < m_score[right-1] )
			return right;

		int mid = (left+right)/2;
		if ( score > m_score[mid] )
			return binarySearch(left, mid, score);
		else 
			return binarySearch(mid, right, score);
	}

private:
	int m_size;
	vector<_T> m_storage;
	vector<_S> m_score;

	set<_T> m_keySet;
};